/*
 * Copyright (c) 1985 Corporation for Research and Educational Networking
 * Copyright (c) 1988 University of Illinois Board of Trustees, Steven
 *              Dorner, and Paul Pomes
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Corporation for
 *      Research and Educational Networking (CREN), the University of
 *      Illinois at Urbana, and their contributors.
 * 4. Neither the name of CREN, the University nor the names of their
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE TRUSTEES AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE TRUSTEES OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef lint
static char RcsId[] = "@(#)$Id: commands.c,v 1.85 1995/06/28 20:32:53 p-pomes Exp $";
#endif /* !lint */

#include "protos.h"

#ifdef __STDC__
#include <stdarg.h>
#else
#include <varargs.h>
#endif
#include <netdb.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>

static ARG *s_args = NULL;

static char *RandomString (int);
static int UserMatch (char *);
static void DoDelete (ARG *);
static void DoFields (ARG *); 
static void DoId (ARG *);
static void DoInfo (ARG *);
static void DoLogin (ARG *);
static void DoAnswer (ARG *);
static void DoLogout (ARG *);
static void DoXLogin (ARG *);
static void DoFwtkLogin (char *);
static void DoGssLogin(char *);
static void DoKrbLogin (int, char *);
static void DoStatus (ARG *);
static void DumpOutput();
static void NotImplemented (ARG *);
static int OkByEmail (QDIR, char *);
static int OkFields (ARG *);
static int OpenTempOut();
static void PrintCommand (int, ARG *);
static void SleepTil (INT32);

#ifdef NO_STRSTR
char *strstr (char *, char *);
#endif /* NO_STRSTR */

extern FILE *Input;		/* qi.c */
extern FILE *Output;		/* qi.c */
extern int InputType;		/* qi.c */
extern char Foreign[80];	/* qi.c */
extern char CommandText[];	/* language.l */
extern char *CommandSpot;	/* language.l */
extern int Daemon;		/* qi.c */
extern int Quiet;		/* qi.c */
extern int LocalUser;		/* qi.c */
extern char *DBState;		/* qi.c */
extern char Revision[];		/* version.c */

#ifdef EMAIL_AUTH
extern struct hostent TrustHp;	/* qi.c */
#endif /* EMAIL_AUTH */

FILE *TempOutput = NULL;
static char *TFile = NULL;

int  DiscardIt;
void (*CommandTable[]) (ARG *) =
{
    DoQuery,
    DoChange,
    DoLogin,
    DoAnswer,
    DoLogout,
    DoFields, 
    DoAdd,
    DoDelete,
    DoSet,
    DoQuit,
    DoStatus,
    DoId,
    DoHelp,
    DoAnswer,
    DoInfo,
    DoAnswer,
    DoXLogin,
    (void (*)(ARG *)) 0
};

QDIR User = NULL;		/* the currently logged in user */
int  AmHero = 0;		/* is currently logged in user the Hero? */
int  AmOpr = 0;			/* is currently logged in user the Operator? */
char *UserAlias = NULL;		/* the alias of the currently logged in user */
int  UserEnt = 0;		/* entry number of current user */
char *ClaimUser = NULL;		/* the identity the user has claimed, but not
				 * yet verified */
char *Challenge = NULL;		/* challenge string */
int  OkResponse = 0;		/* acceptable responses to a challenge */
int  State = S_IDLE;

/*
 * Add a value to an argument list
 */
void
AddValue(value, ttype)
    char *value;
    int  ttype;
{
    static ARG *LastArg = NULL;

    if (value == NULL)
	value = "";
    if (!s_args) {
	LastArg = s_args = FreshArg();
	LastArg->aFirst = strdup(value);
	LastArg->aType = ttype;
    }
    else {
	if (LastArg->aType == VALUE && ttype & EQUAL)
	    LastArg->aType |= ttype;
	else if (LastArg->aType & EQUAL && !(LastArg->aType & VALUE2)) {
	    LastArg->aType |= VALUE2;
	    LastArg->aSecond = strdup(value);
	}
	else {
	    LastArg->aNext = FreshArg();
	    LastArg = LastArg->aNext;
	    LastArg->aType = ttype;
	    LastArg->aFirst = strdup(value);
	}
    }
}

/*
 * Complain about extraneous junk
 */
void
Unknown(junk)
    char *junk;
{
    fprintf(Output, "%d:%s:%s\n", LR_NOCMD, junk, "Command not recognized.");
}

/*
 * Execute a command
 */
void
DoCommand(cmd)
    int  cmd;
{
#define MAX_SYSLOG 100
    char c;
    int  which = cmd == C_CLEAR ? 6 : MAX_SYSLOG;

    c = CommandText[which];
    CommandText[which] = '\0';
    *CommandSpot = '\0';
    if (*CommandText && strstr(CommandText, "password=") == NULL)
	IssueMessage(LOG_INFO, "%s", CommandText);
    CommandText[which] = c;
    if (!Quiet && (InputType == IT_FILE || InputType == IT_PIPE || OP_VALUE(ECHO_OP)))
	PrintCommand(cmd, s_args);	/* echo the cmd */

    if (Daemon && GetState()) {
	fprintf(Output, "555:Database shut off (%s).\n", DBState);
	exit(0);
    }
    if (TempOutput == NULL && !OpenTempOut()) {
	fprintf(Output, "%d:couldn't open temp file.\n", LR_INTERNAL);
	exit(0);
    }
    if (State == S_E_PENDING) {
	if (DiscardIt || cmd != C_ANSWER && cmd != C_CLEAR && cmd != C_EMAIL) {
	    DoReply(-LR_NOANSWER, "Expecting answer, clear, or email; login discarded.");
	    State = S_IDLE;
	    free(ClaimUser);
	    ClaimUser = NULL;
	    free(Challenge);
	    Challenge = NULL;
	}
    }
    if (DiscardIt)
	DoReply(LR_SYNTAX, "Command not understood.");
    else {
	/* (*CommandTable[cmd - 1]) (s_args); */
	void (*cmdP) (ARG *);
	
	if ((cmdP = *CommandTable[cmd - 1]) != NULL)
	    (*cmdP) (s_args);
	else
	    DoReply(LR_SYNTAX, "Command not understood.");
    }

    DumpOutput();

  done:
    FreeArgs(s_args);
    s_args = NULL;
    DiscardIt = 0;
}

/*
 * Free the argument list
 */
void
FreeArgs(arg)
    ARG *arg;
{
    ARG *narg;

    if (arg)
	for (narg = arg->aNext; arg; arg = narg) {
	    narg = arg->aNext;
	    if (arg->aFirst)
		free(arg->aFirst);
	    if (arg->aSecond)
		free(arg->aSecond);
	    free((char *) arg);
	    arg = NULL;
	}
}

/*
 * create a fresh argument structure
 */
ARG *
FreshArg()
{
    ARG *arg;

    arg = (ARG *) malloc(sizeof (ARG));
    memset((void *) arg, (char) 0, sizeof (ARG));
    return (arg);
}

/*
 * status--give the database status
 */
/*ARGSUSED */
static void
DoStatus(arg)
    ARG *arg;
{
    char banner[MAXPATHLEN], vstr[MAXSTR];
    FILE *bfp;

    (void) sprintf(banner, "%s.bnr", Database);
    if (bfp = fopen(banner, "r")) {
	(void) sprintf(vstr, "ph2ldap adaptor %s", Revision);
	DoReply(LR_PROGRESS, vstr);
	while (fgets(banner, sizeof (banner) - 1, bfp)) {
	    char *nl = strchr(banner, '\n');

	    if (nl)
		*nl = '\0';
	    DoReply(LR_PROGRESS, banner);
	}
	(void) fclose(bfp);
    }
    if (ReadOnly)
	DoReply(LR_RONLY, "Database ready, read only (%s).", DBState);
    else
    {
	(void) sprintf(vstr, "ph2ldap adaptor %s", Revision);
	DoReply(LR_PROGRESS, vstr);
	DoReply(LR_OK, "Database ready.");
    }
}

/*
 * id--this command is a no-op; the client issues it only to put
 * the name of the calling user into the nameserver's logfiles.
 */
/*ARGSUSED */
static void
DoId(arg)
    ARG *arg;
{
    DoReply(LR_OK, "Thanks.");
}


void
DoAdd(arg)
    ARG *arg;
{
   NotImplemented(arg);
}


void
DoChange(arg)
    ARG *arg;
{
   NotImplemented(arg);
}


void
DoHelp(arg)
    ARG *arg;
{
   NotImplemented(arg);
}

/*
 * quit
 */
/*ARGSUSED */
void
DoQuit(arg)
    ARG *arg;
{
    fprintf(Output, "%d:%s\n", LR_OK, "Bye!");
    fflush(Output);
    IssueMessage(LOG_INFO, "Done 0");
    closelog();
   
    /* close any existing Ldap connections */

    ExitLdap();

    
    exit(1);
}

/*
 * info,  N.B., the ordering of the #define's for the authenticate section
 * is the order that the server prefers the clients to use.  If a different
 * order is preferred, the code sections must be moved around.
 */
/*ARGSUSED */
static void
DoInfo(arg)
    ARG *arg;
{
    short n = 0;

    DoReply(-LR_OK, "%d:mailfield:%s", ++n, MAILFIELD);
    DoReply(-LR_OK, "%d:maildomain:%s", ++n, MAILDOMAIN);
    DoReply(-LR_OK, "%d:administrator:%s", ++n, ADMIN);
    DoReply(-LR_OK, "%d:mailbox:%s", ++n, MAILBOX);
    DoReply(LR_OK, "Ok.");

}

/*
 * Not implemented
 */
static void
NotImplemented(arg)
    ARG *arg;
{
    DoReply(500, "%s:command not implemented.", arg->aFirst);
}

/*
 * make a reply to a command
 */
/*VARARGS2 */
void
#ifdef __STDC__
DoReply(int code, char *fmt,...)
#else /* !__STDC__ */
DoReply(code, fmt, va_alist)
    int  code;
    char *fmt;
    va_dcl

#endif /* __STDC__ */
{
    char scratchFormat[256];
    char buf[4096];
    va_list args;

    (void) sprintf(scratchFormat, "%d:%s\n", code, fmt);

#ifdef __STDC__
    va_start(args, fmt);
#else /* !__STDC__ */
    va_start(args);
#endif /* __STDC__ */
    vsprintf(buf, scratchFormat, args);
    va_end(args);
    if (TempOutput == NULL && !OpenTempOut()) {
	fprintf(Output, "%d:couldn't open temp file.\n", LR_INTERNAL);
	return;
    }
    if (fprintf(TempOutput, "%s", buf) == EOF) {
	IssueMessage(LOG_ERR, "DoReply: fprintf: %s", strerror(errno));
	fprintf(Output, "%d:couldn't write to temp file.\n", LR_INTERNAL);
	exit(0);
    }
    /* IssueMessage(LOG_DEBUG, "%s", buf); */
}


/*
 * Select which authentication method to use.
 */
static void
DoXLogin(arg)
    ARG *arg;
{

  NotImplemented(arg);

}


/*
 * Identify user the old-fashioned way.
 */
static void
DoLogin(arg)
    ARG *arg;
{
   NotImplemented(arg);
}


/*
 * handle the answer to a challenge
 */
#define WAIT_SECS 1
static void
DoAnswer(arg)
    ARG *arg;
{
   NotImplemented(arg);
}

/*
 * sleep til a given time
 */
static void
SleepTil(xtime)
    INT32 xtime;
{
    unsigned span;

    span = xtime - time((INT32 *) 0);
    if (0 < span && span < 10000)
	sleep(span);
}


/*
 * de-identify the current user
 */
static void
DoLogout(arg)
    ARG *arg;
{
   NotImplemented(arg);
}


/*
 * list fields
 */
static void
DoFields(arg)
    ARG *arg;
{
   NotImplemented(arg);
}



/*
 * delete entries
 */
static void
DoDelete(arg)
    ARG *arg;
{
   NotImplemented(arg);
}

/*
 * open a temp file for output
 */
static int
OpenTempOut()
{
    if (TFile == NULL) {
	TFile = strdup(TEMPFILE);
	mktemp(TFile);
    }
    if ((TempOutput = fopen(TFile, "w+")) == NULL) {
	IssueMessage(LOG_ERR, "OpenTempOut: fopen(%s) failed: %s",
		     TFile, strerror(errno));
	free(TFile);
	TFile = NULL;
	return (0);
    }
    unlink(TFile);

    return (1);
}

/*
 * Dump a the stuff in TFile to output
 */
static void
DumpOutput()
{
    int  c;

    rewind(TempOutput);		/* back to the beginning */
    {
	while ((c = getc(TempOutput)) != EOF)
	    putc(c, Output);
    }
    fclose(TempOutput);		/* close; already unlinked */
    TempOutput = NULL;
    fflush(Output);
}

/*
 * print the current command
 */
static void
PrintCommand(cmd, arg)
    int  cmd;
    ARG *arg;
{
    fprintf(Output, "%d: %s", LR_ECHO, arg->aFirst);
    for (arg = arg->aNext;
	 arg;
	 arg = arg->aNext) {
	putc(' ', Output);
	if (arg->aType == RETURN)
	    fputs(cmd == C_QUERY ? "return" : "make", Output);
	else {
	    if (arg->aType & VALUE)
		fputs(arg->aFirst, Output);
#ifdef DO_TILDE
	    if (arg->aType & TILD_E)
		putc('~', Output);
	    else
#endif
	    if (arg->aType & EQUAL)
		putc('=', Output);
	    if (arg->aType & VALUE2)
		fputs(arg->aSecond, Output);
	}
    }
    putc('\n', Output);
}








/*
 * generate a random string
 */
static char *
RandomString(byteCount)
    int  byteCount;
{
    static char string[MAXSTR];
    char *cp;
    static int seeded = 0;

    if (!seeded) {
	seeded = 1;
	srand((int) time((INT32 *) NULL) ^ getpid());
    }
    for (cp = string; byteCount; cp++, byteCount--)
	*cp = (rand() & 0x3f) + 0x21;

    return (string);
}


#ifdef EMAIL_AUTH
/*
 * figure out if a user is ok by his email address
 */
static int
OkByEmail(User, username)
    QDIR User;
    char *username;
{
    char buf[256];
    char *email = FINDVALUE(User, F_EMAIL);
    char *new, *spnt, *epnt;
    int  result;

    /*
     * Fix up email field by omitting leading whitespace and
     * terminating it at the first white space.
     */
    new = spnt = strdup(email);
    while (isspace(*spnt))
	spnt++;
    epnt = spnt;
    while (*epnt && !isspace(*epnt))
	epnt++;
    *epnt = '\0';
    if (!TrustHp.h_name || !*spnt)
	result = 1;
    else {
	(void) sprintf(buf, "%s@%s", username, TrustHp.h_name);
	result = stricmp(spnt, buf);
    }
    free(new);
    if (result)
	DoReply(LR_NOEMAIL, "You can't login that way.");
    return (!result);
}

#endif


int 
IssueMessage(int severity, char *format, ...)
{

    return 1;
}





/*-
 * Copyright (c) 1990 The Regents of the University of California.
 * All rights reserved.
 *
 * This code is derived from software contributed to Berkeley by
 * Chris Torek.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that: (1) source distributions retain this entire copyright
 * notice and comment, and (2) distributions including binaries display
 * the following acknowledgement:  ``This product includes software
 * developed by the University of California, Berkeley and its contributors''
 * in the documentation or other materials provided with the distribution
 * and in all advertising materials mentioning features or use of this
 * software. Neither the name of the University nor the names of its
 * contributors may be used to endorse or promote products derived
 * from this software without specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

#if defined(LIBC_SCCS) && !defined(lint)
static char sccsid[] = "@(#)strstr.c	5.1 (Berkeley) 5/15/90";

#endif /* LIBC_SCCS and not lint */

#ifdef NO_STRSTR

/*
 * Find the first occurrence of find in s.
 */
char *
strstr(s, find)
    register char *s, *find;
{
    register char c, sc;
    register int len;

    if ((c = *find++) != 0) {
	len = strlen(find);
	do {
	    do {
		if ((sc = *s++) == 0)
		    return ((char *) 0);
	    } while (sc != c);
	} while (strncmp(s, find, len) != 0);
	s--;
    }
    return ((char *) s);
}

#endif
