/*
 * Copyright (c) 1985 Corporation for Research and Educational Networking
 * Copyright (c) 1988 University of Illinois Board of Trustees, Steven
 *		Dorner, and Paul Pomes
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the Corporation for
 *	Research and Educational Networking (CREN), the University of
 *	Illinois at Urbana, and their contributors.
 * 4. Neither the name of CREN, the University nor the names of their
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE TRUSTEES AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE TRUSTEES OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * @(#)$Id: protos.h,v 1.37 1995/06/28 20:35:18 p-pomes Exp $
 */

#ifndef PROTOS_H
#define PROTOS_H
#include <stdio.h>
#include <ctype.h>
#include <sys/types.h>
#ifdef __STDC__
# include <stdlib.h>
# include <string.h>
#else /* !__STDC__ */
# include <strings.h>
#endif /* __STDC__ */
#include "bintree.h"
#include "commands.h"
#include "conf.h"
#include "qdb.h"
#include "field.h"
#include "log.h"
#include "options.h"
#include "qi.h"
#include "qiapi.h"
#ifndef __STDC__
MEM_TYPE *malloc(), *realloc();
char	*strchr(), *strrchr(), *strtok();
#endif /* !__STDC__ */
#ifdef NO_STRDUP
char	*strdup();
#endif

/* comment out if fcrypt() bombs on your host */
#ifndef crypt
# define crypt		fcrypt
#endif /* !crypt */

extern int errno;

char	*add_string (char *);	/* lib/strbuild.c - not used */
void	AddValue (char *, int);
int	AliasIsUsed (char *, INT32);
int	AllMeta (char *);
IDX	allocate_leaf();
int	any (int, char *);		/* 1 char -> int */
int	anyof (register char *, register char *);
char	*BadAlias (QDIR, char *);
void	bintree_init (char *);
int	blankline (char *);	/* lib/str.c - not used */
int	CanAddEntries();
int	CanChange (QDIR, FDESC *);
int	CanDelete (QDIR);
int	CanLookup (FDESC *, char *);
int	CanSee (QDIR, FDESC *, int);
int	ChangeDir (QDIR *, FDESC *, char *);
void	cleanup (int);
void	close_tree();
int	countc (char *, int);	/* 2 char -> int, str.c - not used */
int	CurrentDate();
int	CurrentIndex();
int	dbd_init (char *);
int	dbi_init (char *);
int	decode (unsigned char *, unsigned char *);
int	qidelete (char *);
int	deletestr (char *, int);
int	dnext_ent (INT32);
void	DoAdd (ARG *);
void	DoChange (ARG *);
void	DoCommand (int);
void	DoHelp (ARG *);
INT32	*DoLookup (ARG *);
INT32	*do_lookup (char **, INT32 *);
void	DoQuery (ARG *);
void	DoQuit (ARG *);
void	DoReply (int, char *, ...);
void	DoSet (ARG *);
int	DoSysLog (int);
int	encode (char *, char *, int);
int	ent_dead();
void	expand (LEAF *, int, ITEM *);
char	*FascistCheck (QDIR, char *, char *);
INT32	*find_all (char *);
FDESC	*FindFD (char *);
FDESC	*FindFDI (int);
/* int	FindField (QDIR, int); */
INT32	*findstr (char *);
void	FreeArgs (ARG *);
void	free_ary (INT32 **);
void	FreeDir (QDIR *);
ARG	*FreshArg ();
QDIR	GetAliasDir (char *);
char	**getdata (QDIR *);
DREC	*getdirent (INT32);
void	get_dir_head();
INT32	*get_dir_ptrs (INT32);
int	GetFieldConfig();
#ifdef VMS
int	GetGood (char *, int, int);
#else
int	GetGood (char *, int, FILE *);
#endif /*VMS*/
int	GetState();
void	get_tree_head();
char	*getword (char *, char *);	/* lib/util.c - not used */
int	GonnaRead (char *);
int	GonnaWrite (char *);
int	icopy (char *, char *);
INT32	ihash (char *, INT32, INT32);
void	InitializeLogging();
void	InitializeOptions();
void	init_string();
void	insert (char *, IDX);
INT32	*intersect (INT32 *, INT32 *);
int	issub (char *, char *);
int	IssueMessage (int, char *fmt, ...);
int	length (INT32 *);
int	Plength (INT32 **);
char	*ltrunc (char *);		/* lib/str.c - not used */
void	make_lookup (char *, int);
char	*make_str (char *);
INT32	*merge (INT32 *, INT32 *);
void	mkargv (int *, char **, char *);
int	new_ent();
int	next_ent (INT32);
char	*PasswordOf (QDIR);
char	*phonemify (char *);
int	putarry (struct iindex *, INT32 *, INT32);
int	putdata (char **);
void	putdirent (INT32, DREC *);
void	put_dir_head();
int	putstr (char *, int);
void	put_tree_head();
int	quoted (char *);
void	read_index (char *);
void	read_leaf (IDX, LEAF *);
int	RateAKey (char *);
int	RemoveSpecials (char *);
int	search (char *, LEAF *, int *);
void	set_date (int);
void	SetDeleteMark();
void	store_ent();
int	strcate (char *, char *);
int	strcpc (register char *, register char *);
char	*strecat (char *, char *); /* lib/str.c - not used */
char	*strecpy (char *, char *); /* lib/str.c - not used */
int	stricmp (char *, char *);
int	strincmp (char *, char *, int);
char	*strlcpy (char *, char *);
void	Unknown (char *);
void	Unlock (char *);
void	unmake_lookup (char *, int);
int	UserCanChange (QDIR);
int	UserCanDelete();
void	write_leaf (LEAF *);
int	ValidQuery (ARG *, int);
int	kdb_db_init();
int	kdb_add_entry (char *);
int	kdb_del_entry (char *);

#endif	/*PROTOS_H*/
